==> fap-0.9.2.tar.bz2: fap-0.9.2/src/audio/converter/quantizer.cpp <==
/*
 *  Copyright (C) 2006-2010 Leandro Nini
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
 */

#include "quantizer.h"

template<typename O>
quantizerFixed<O>::quantizerFixed(const unsigned int fract) {

	_random[0][0]=3686734;
	_random[0][1]=86526882;
	_random[1][0]=268268;
	_random[1][1]=7628932;

	FXTRACE((10, "quantizerFixed %d bytes\n", sizeof(O)));
	_scalebits=fract+1-(sizeof(O)<<3);
	_mask=(1L<<_scalebits)-1;
	_clip=(1L<<fract);
}

template quantizerFixed<FXuchar>::quantizerFixed(const unsigned int fract);
template quantizerFixed<FXshort>::quantizerFixed(const unsigned int fract);

template<typename O>
inline FXint quantizerFixed<O>::get32(const FXint sample, const unsigned int channel) {

	const int r1=fxrandom(_random[channel][0])&_mask;
	const int r2=fxrandom(_random[channel][1])&_mask;

	// Dither
	FXint output=sample+
		r1+r2+
		(1L<<(_scalebits-1));

	// Clip
	if (output>_clip-1)
		output=_clip-1;
	else if (output<-_clip)
		output=-_clip;

	return output;
}

template<>
FXuchar quantizerFixed<FXuchar>::get(const FXint sample, const unsigned int channel) {

	return (FXuchar)(get32(sample, channel)>>_scalebits)+128;
}

template<>
FXshort quantizerFixed<FXshort>::get(const FXint sample, const unsigned int channel) {

	return (FXshort)(get32(sample, channel)>>_scalebits);
}

/******************************************************************************/

template<typename O>
quantizerFloat<O>::quantizerFloat() {

	_random[0][0]=3686734;
	_random[0][1]=86526882;
	_random[1][0]=268268;
	_random[1][1]=7628932;

	FXTRACE((10, "quantizerFloat %d bytes\n", sizeof(O)));
}

template quantizerFloat<FXuchar>::quantizerFloat();
template quantizerFloat< FXshort>::quantizerFloat();

template<typename O>
inline FXint quantizerFloat<O>::get32(const FXfloat sample, const unsigned int channel, const FXint max) {

	const FXfloat r1=(FXfloat)(FXint)fxrandom(_random[channel][0])/(2147483648.0f);
	const FXfloat r2=(FXfloat)(FXint)fxrandom(_random[channel][1])/(2147483648.0f);

	// Dither
	FXint output=(FXint)(sample*(FXfloat)max+
		r1+r2+
		0.5f);

	// Clip
	if (output>max-1)
		output=max-1;
	else if (output<-max)
		output=-max;

	return output;
}

template<>
FXuchar quantizerFloat<FXuchar>::get(const FXfloat sample, const unsigned int channel) {

	return (FXuchar)(get32(sample, channel, 128))+128;
}

template<>
FXshort quantizerFloat<FXshort>::get(const FXfloat sample, const unsigned int channel) {

	return (FXshort)(get32(sample, channel, 32768));
}

==> fap-0.10.0.tar.bz2: fap-0.10.0/src/audio/converter/quantizer.cpp <==
/*
 *  Copyright (C) 2006-2013 Leandro Nini
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
 */

#include "quantizer.h"

#include <limits>

template<typename O>
quantizerFixed<O>::quantizerFixed(const unsigned int fract) {

	_random[0][0]=3686734;
	_random[0][1]=86526882;
	_random[1][0]=268268;
	_random[1][1]=7628932;

	FXTRACE((10, "quantizerFixed %d bytes\n", sizeof(O)));
	_scalebits=fract+1-(sizeof(O)<<3);
	_mask=(1L<<_scalebits)-1;
	_clip=(1L<<fract);
}

template quantizerFixed<FXuchar>::quantizerFixed(const unsigned int fract);
template quantizerFixed<FXshort>::quantizerFixed(const unsigned int fract);

template<typename O>
inline FXint quantizerFixed<O>::get32(const FXint sample, const unsigned int channel) {

	const FXuint r1=fxrandom(_random[channel][0])&_mask;
	const FXuint r2=fxrandom(_random[channel][1])&_mask;

	// Dither
	FXint output=sample+
		r1+r2+
		(1L<<(_scalebits-1));

	// Clip
	if (output>_clip-1)
		output=_clip-1;
	else if (output<-_clip)
		output=-_clip;

	return output;
}

template<>
FXuchar quantizerFixed<FXuchar>::get(const FXint sample, const unsigned int channel) {

	return (FXuchar)(get32(sample, channel)>>_scalebits)+128;
}

template<>
FXshort quantizerFixed<FXshort>::get(const FXint sample, const unsigned int channel) {

	return (FXshort)(get32(sample, channel)>>_scalebits);
}

/******************************************************************************/

template<typename O>
quantizerFloat<O>::quantizerFloat() {

	_random[0][0]=3686734;
	_random[0][1]=86526882;
	_random[1][0]=268268;
	_random[1][1]=7628932;

	FXTRACE((10, "quantizerFloat %d bytes\n", sizeof(O)));
}

template quantizerFloat<FXuchar>::quantizerFloat();
template quantizerFloat<FXshort>::quantizerFloat();

template<typename O>
inline FXint quantizerFloat<O>::get32(const FXfloat sample, const unsigned int channel, const FXint max) {

	const FXfloat r1=(FXfloat)fxrandom(_random[channel][0])/(FXfloat)std::numeric_limits<FXuint>::max();
	const FXfloat r2=(FXfloat)fxrandom(_random[channel][1])/(FXfloat)std::numeric_limits<FXuint>::max();

	// Dither
	FXint output=(FXint)(sample*(FXfloat)max+
		r1+r2+
		0.5f);

	// Clip
	if (output>max-1)
		output=max-1;
	else if (output<-max)
		output=-max;

	return output;
}

template<>
FXuchar quantizerFloat<FXuchar>::get(const FXfloat sample, const unsigned int channel) {

	return (FXuchar)(get32(sample, channel, 128))+128;
}

template<>
FXshort quantizerFloat<FXshort>::get(const FXfloat sample, const unsigned int channel) {

	return (FXshort)(get32(sample, channel, 32768));
}
